--==Combine Ring/Hyper Ring Monitor by Lach==--

function A_CombineRingBox(mo)
	if mo and mo.target and mo.target.player
		mo.target.combine = true
		mo.target.hyperring = 0
		S_StartSound(mo.target, sfx_hyprng)
		MP_giveEffect(mo.target.player, ef_hypercombi)
	end
end

function A_HyperRingBox(mo)
	if mo and mo.target and mo.target.player
		mo.target.hyperring = 1
		mo.target.combine = false
		S_StartSound(mo.target, sfx_hyprng)
		MP_giveEffect(mo.target.player, ef_hypercombi)
	end
end

MP_buildMonitor(MT_COMBINERINGMONITOR, MT_COMBINERINGICON, S_CRINGTV, S_CRINGPOP1, SPR_CRBX, SPR_YLTV, A_CombineRingBox)
MP_buildMonitor(MT_HYPERRINGMONITOR, MT_HYPERRINGICON, S_HRINGTV, S_HRINGPOP1, SPR_HRBX, SPR_YLTV, A_HyperRingBox)

states[S_CRNG] = {
	sprite = SPR_CRNG,
	frame = A|FF_ANIMATE,
	var1 = 9,
	var2 = 2,
	nextstate = S_CRNG
}

states[S_HRNG] = {
	sprite = SPR_HRNG,
	frame = A|FF_ANIMATE,
	var1 = 7,
	var2 = 3,
	nextstate = S_HRNG
}

//Sets up players to store their ring count in a new variable and only release one flingring when hurt holding a combine ring/hyper ring

addHook("MobjDamage", function(mo)
	if mo and mo.player and not mo.player.powers[pw_shield] and mo.player.rings > 0
		if mo.combine
			mo.ringcount = mo.player.rings> 200 and 200 or mo.player.rings
			mo.player.rings = 1
			if combineweapons
				mo.pickups = mo.player.ringweapons
				mo.autorings = mo.player.powers[pw_automaticring]
				mo.bouncerings = mo.player.powers[pw_bouncering]
				mo.bombrings = mo.player.powers[pw_explosionring]
				mo.minerings = mo.player.powers[pw_grenadering]
				mo.scatterrings = mo.player.powers[pw_scatterring]
				mo.railrings = mo.player.powers[pw_railring]
				mo.looprings = mo.player.powers[pw_infinityring]
			end
		elseif mo.hyperring
			mo.ringcount = FixedSqrt((mo.player.rings)*FRACUNIT)/FRACUNIT or 1
			mo.player.rings = ($-1)/mo.ringcount + 1
			mo.hyperring = mo.player.rings
		end
	end
end, MT_PLAYER)

//Gets rid of dropped match weapons if the variables permit

local function ridMatch(mo)
	if combineweapons and mo and mo.target and mo.target.combine
		P_RemoveMobj(mo)
	end
end

for i = MT_BOUNCERING, MT_GRENADEPICKUP
	addHook("MobjThinker", function(mo)
		ridMatch(mo)
	end, i)
end

//Flingrings are modified if their target player holds a combine ring/hyper ring; also manages player variables and sounds

addHook("MobjThinker", function(mo)
	if not mo or not mo.valid return end
	if mo and not mo.locked and mo.target
		if mo.target.ringcount
			mo.momx = $1 * 2
			mo.momy = $1 * 2
			mo.momz = $1 * 3
			mo.ringvalue = mo.target.ringcount - 1
			if mo.target.combine
				mo.state = S_CRNG
				mo.height = $1*2
				mo.radius = $1*2
				mo.fuse = 6*TICRATE
				mo.target.combine = false
				mo.target.ringcount = nil
				MP_loseEffect(mo.target.player, ef_hypercombi)
			elseif mo.target.hyperring
				mo.state = S_HRNG
				mo.height = 3*$1/2
				mo.radius = 3*$1/2
				mo.target.hyperring = $ - 1
				if not mo.target.hyperring
					mo.target.ringcount = nil
					MP_loseEffect(mo.target.player, ef_hypercombi)
				end
			end
			if mo.target.pickups
				if combineweapons
					mo.pickups = mo.target.pickups
					mo.autorings = mo.target.autorings
					mo.bouncerings = mo.target.bouncerings
					mo.bombrings = mo.target.bombrings
					mo.minerings = mo.target.minerings
					mo.scatterrings = mo.target.scatterrings
					mo.railrings = mo.target.railrings
					mo.looprings = mo.target.looprings
				end
				mo.target.pickups = nil
				mo.target.autorings = nil
				mo.target.bouncerings = nil
				mo.target.bombrings = nil
				mo.target.minerings = nil
				mo.target.scatterrings = nil
				mo.target.railrings = nil
				mo.target.looprings = nil
			end
			mo.locked = true
		else
			mo.locked = true
		end
	end
end, MT_FLINGRING)

//Gives players the stored ring count when the combine flingring is picked up

addHook("MobjDeath", function(mo, char, char)
	if mo and mo.ringvalue and char and char.player
		mo.scale = $1*2
		char.player.rings = $1 + mo.ringvalue
		if mo.pickups and combineweapons
			char.player.ringweapons = $1 | mo.pickups
			char.player.powers[pw_automaticring] = $1 + mo.autorings
			char.player.powers[pw_bouncering] = $1 + mo.bouncerings
			char.player.powers[pw_explosionring] = $1 + mo.bombrings
			char.player.powers[pw_grenadering] = $1 + mo.minerings
			char.player.powers[pw_scatterring] = $1 + mo.scatterrings
			char.player.powers[pw_railring] = $1 + mo.railrings
			char.player.powers[pw_infinityring] = $1 + mo.looprings
		end
	end
end, MT_FLINGRING)

//Spawns flingrings (and match rings, if enabled) when the combine ring's fuse is up

local function flingWeapon(mo, val, weapon, panel, ring)
	if (mo.pickups & weapon)
		local fling = P_SpawnMobj(mo.x, mo.y, mo.z, panel)
		fling.momx = P_RandomRange(-4,4)*FRACUNIT
		fling.momy = P_RandomRange(-4,4)*FRACUNIT
		fling.momz = P_RandomRange(-8,8)*FRACUNIT
		fling.fuse = 12*TICRATE
		fling.target = mo.target
		fling.flags = $1 & ~MF_NOGRAVITY & ~MF_NOCLIPHEIGHT
		val = $1 - 5
	end
	if val > 0
		local fling = P_SpawnMobj(mo.x, mo.y, mo.z, ring)
		fling.momx = P_RandomRange(-4,4)*FRACUNIT
		fling.momy = P_RandomRange(-4,4)*FRACUNIT
		fling.momz = P_RandomRange(-8,8)*FRACUNIT
		fling.fuse = 12*TICRATE
		fling.target = mo.target
		fling.health = val
		fling.flags = $1 & ~MF_NOGRAVITY & ~MF_NOCLIPHEIGHT
	end
end

addHook("MobjRemoved", function(mo)
	if mo and mo.ringvalue and mo.state == S_CRNG //and not (mo.state >= mo.info.deathstate and mo.state <= mo.info.deathstate + 15)
		for i = 1, mo.ringvalue, 1
			local ring = P_SpawnMobj(mo.x, mo.y, mo.z, MT_FLINGRING)
			ring.momx = P_RandomRange(-4,4)*FRACUNIT
			ring.momy = P_RandomRange(-4,4)*FRACUNIT
			ring.momz = P_RandomRange(-8,8)*FRACUNIT
			ring.fuse = 8*TICRATE
			ring.target = mo.target
		end
		if mo.pickups and combineweapons
			flingWeapon(mo, mo.autorings, RW_AUTO, MT_AUTOPICKUP, MT_AUTOMATICRING)
			flingWeapon(mo, mo.bouncerings, RW_BOUNCE, MT_BOUNCEPICKUP, MT_BOUNCERING)
			flingWeapon(mo, mo.bombrings, RW_EXPLODE, MT_EXPLODEPICKUP, MT_EXPLOSIONRING)
			flingWeapon(mo, mo.minerings, RW_GRENADE, MT_GRENADEPICKUP, MT_GRENADERING)
			flingWeapon(mo, mo.scatterrings, RW_SCATTER, MT_SCATTERPICKUP, MT_SCATTERRING)
			flingWeapon(mo, mo.railrings, RW_RAIL, MT_RAILPICKUP, MT_RAILRING)
			if mo.looprings > 0
				local fling = P_SpawnMobj(mo.x, mo.y, mo.z, MT_INFINITYRING)
				fling.momx = P_RandomRange(-4,4)*FRACUNIT
				fling.momy = P_RandomRange(-4,4)*FRACUNIT
				fling.momz = P_RandomRange(-8,8)*FRACUNIT
				fling.fuse = 12*TICRATE
				fling.target = mo.target
				fling.health = mo.looprings
				fling.flags = $1 & ~MF_NOGRAVITY & ~MF_NOCLIPHEIGHT
			end
		end
	end
end, MT_FLINGRING)

//HUD

local combineicon
local hypericon

local function findDigit(num)
	return num > 1000 and 24
	or num > 100 and 16
	or num > 10 and 8
	or 0
end

local cmbi_data = {
	hudfunc = function(v, player, c)
	
		if player.mo and (player.mo.combine or player.mo.hyperring)

			local xpos, ypos
			local flags = 0

			if not combineicon then combineicon = v.cachePatch("HUD_CRBX") end
			if not hypericon then hypericon = v.cachePatch("HUD_HRBX") end

			local icon = player.mo.hyperring and hypericon or combineicon

			flags = V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER
			if CV_FindVar("timerres").value == 2
				xpos = 86 -findDigit(player.rings)
			else
				xpos = 98
			end	
				
			ypos = 38

			v.drawScaled(xpos<<FRACBITS, ypos<<FRACBITS, FRACUNIT, icon, flags)
		end
	end	
	
}

effectList[ef_hypercombi] = cmbi_data
