function A_CactiShield(actor)
	if not actor.target or not actor.target.player
		print("ERROR: A_CactiShield: Power up has no target.")
		return
	end
	local p = actor.target.player
	MP_giveShield(p, SH_CACTUSSHIELD)
end

local function spawnCshieldoverlays(mo)
	local p = mo.player
	local cactis_front = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_OVERLAY)
	cactis_front.state = S_CACTISHIELD_FRONT1
	cactis_front.target = p.mo
	p.mo.cactis_front = cactis_front

	local cactis_back = P_SpawnMobj(p.mo.x, p.mo.y, p.mo.z, MT_OVERLAY)
	cactis_back.state = S_CACTISHIELD_BACK1
	cactis_back.target = p.mo
	p.mo.cactis_back = cactis_back
	
	-- since this is the get function, might as well get this out of the way!
	
end

-- spike burst function to save us some time on instances where we'll need it

local function cactiSpikeBurst(mo)

	local h_angle = 0
	local v_angle = 0
	local dist = 32
	local frame = 0

	for i=1,26

		local s_x = mo.x+ dist*FixedMul(cos(h_angle), cos(v_angle))
		local s_y = mo.y+ dist*FixedMul(sin(h_angle), cos(v_angle))
		local s_z = (mo.z+mo.height/2)+ dist* sin(v_angle)

		local spike = P_SpawnMobj(s_x, s_y, s_z, MT_CACTISPIKE)
		spike.frame = frame
		spike.angle = h_angle
		spike.target = mo -- dont damage self rofl
		spike.fuse = TICRATE/3
		spike.momx = 32 * FixedMul(cos(h_angle), cos(v_angle))
		spike.momy = 32 * FixedMul(sin(h_angle), cos(v_angle))
		spike.momz = 32 * sin(v_angle)

		h_angle = $+ANGLE_45

		if i == 8
			v_angle = ANGLE_45
			frame = 1
		elseif i == 16
			v_angle = -ANGLE_45
			frame = 2
		elseif i == 24
			v_angle = ANGLE_90
			frame = 3
		elseif i == 25
			v_angle = -ANGLE_90
			frame = 4
		end
	end

	P_NukeEnemies(mo, mo, 192*FRACUNIT)
	S_StartSound(mo, sfx_spkdth)
	mo.RRZ_nuke = true


end

-- main thinker for the shield and the player!

local function cacti_main(mo)
	if mo.player and mo.player.valid
		local p = mo.player
		
		mo.RRZ_nuke = nil

		if mo.cactis_front and mo.cactis_front.valid
		and mo.cactis_back and mo.cactis_back.valid
		
			if p.powers[pw_invulnerability]
			or p.powers[pw_super]
			or mo.MP_effects[ef_invisible]
				mo.cactis_front.flags2 = $|MF2_DONTDRAW
				mo.cactis_back.flags2 = $|MF2_DONTDRAW
			else
				mo.cactis_back.flags2 = $ & ~MF2_DONTDRAW
				mo.cactis_front.flags2 = $ & ~MF2_DONTDRAW
			end
		end
		
		if mo.cactis_front and mo.cactis_front.valid
		and mo.cactis_back and mo.cactis_back.valid
			if (p.pflags & PF_SHIELDABILITY)
				mo.cactis_front.flags2 = $|MF2_DONTDRAW
				mo.cactis_back.flags2 = $|MF2_DONTDRAW
				
				mo.RRZ_shieldability = true
			else
				-- play a sound
				if (mo.RRZ_shieldability)
					mo.RRZ_shieldability = nil
					S_StartSound(mo, sfx_s3k64)
				end
				
				mo.cactis_back.flags2 = $ & ~MF2_DONTDRAW
				mo.cactis_front.flags2 = $ & ~MF2_DONTDRAW
			end
		end
		
		if mo.cactis_front and mo.cactis_front.valid
		and mo.cactis_back and mo.cactis_back.valid
			if p.powers[pw_invulnerability]
			or p.powers[pw_super]
			or mo.MP_effects[ef_invisible]
				mo.cactis_front.flags2 = $|MF2_DONTDRAW
				mo.cactis_back.flags2 = $|MF2_DONTDRAW		
			end
		end	
	end
end

local function cacti_superspikehide(mo)
	
	local p = mo.player
	
	if mo.cactis_front and mo.cactis_front.valid
	and mo.cactis_back and mo.cactis_back.valid
						
		if p.powers[pw_invulnerability]
		or p.powers[pw_super]
		or mo.MP_effects[ef_invisible]
			mo.cactis_front.flags2 = $|MF2_DONTDRAW
			mo.cactis_back.flags2 = $|MF2_DONTDRAW
		end
	end	
end

local function cacti_special(mo)
	local p = mo.player
	
	p.pflags = $|(PF_THOKKED|PF_SHIELDABILITY)
	cactiSpikeBurst(mo)
end

local function inSpikeSec(mo)
	local sec = mo.subsector.sector
	local numlines = #sec.lines

	if sec.special == 5
		return true
	end

	for i=1,numlines

		local v = sec.lines[i]
		if v

			if v.frontsector and v.frontsector.special == 5
			or v.backsector and v.backsector.special == 5
				return true
			end
		end
	end
end

local function ccspikesRemove(mo)
	if mo.cactis_front and mo.cactis_front.valid and mo.cactis_back and mo.cactis_back.valid
		P_RemoveMobj(mo.cactis_front)
		P_RemoveMobj(mo.cactis_back)
	end
end

local function cchit(mo)
	cactiSpikeBurst(mo)
	ccspikesRemove(mo)
end

local cshield_data = {
	orbstate = S_PITY1,
	hudico = "CCICON",
	sound = sfx_spkdth,
	get = spawnCshieldoverlays,
	func = cacti_main,
	superfunc = cacti_superspikehide,
	special = cacti_special,
	hit = cchit,
	change = ccspikesRemove,
}
shieldList[SH_CACTUSSHIELD] = cshield_data

-- this damages players that try to hug you if they don't have "protection" ;)

local cshieldtouch = function(mo, touch)

	if not touch return end						-- no item wtf
	if not touch.player return end				-- not even a fucking player
	if not mo.player return end
	if not (touch.player.powers[pw_shield] == SH_CACTUSSHIELD) return end		-- no shield fuck you
	if mo.player.powers[pw_super] return end	-- a
	if touch.shieldcooldown return end	-- spikes down fuck you
	if touch.z > mo.z+mo.height return end		-- too high
	if touch.z+touch.height < mo.z return end	-- too low


	if not (mo.player.powers[pw_shield] == SH_CACTUSSHIELD)				-- avoid useless c stacks overflow
		P_DamageMobj(mo, touch, touch)			-- rape
	end
end


addHook("MobjCollide", cshieldtouch, MT_PLAYER)
addHook("MobjMoveCollide", cshieldtouch, MT_PLAYER)


-- this shield prevents spike damage and direct enemy damage if the spikes are up!

addHook("ShouldDamage", function(mo, inflictor)

	if not inflictor return end

	if mo and mo.valid
	local p = mo.player

		if not p return end

		if p.powers[pw_shield] == SH_CACTUSSHIELD and inflictor and inflictor.valid

			-- case 1: spike OBJECT related damage
			if inflictor.type == MT_SPIKE and inflictor.flags & MF_AMBUSH
				return false
			end
			-- case 2: spike SECTOR related damage
			if inflictor.type == 0 and inSpikeSec(mo)
				return false
			end
			-- case 3: direct enemy attack WHILE SPINNING / JUMPING
			if inflictor.flags & MF_ENEMY and (p.pflags & PF_JUMPED or p.pflags & PF_SPINNING)
				P_DamageMobj(inflictor, mo, mo)
				return false
			end

			-- case 4: against player with cacti shield

			if inflictor.type == MT_PLAYER
			and inflictor.player and inflictor.player.powers[pw_shield] == SH_CACTUSSHIELD
			and not inflictor.player.powers[pw_invulnerability]
				return false
			end

			if inflictor.type == MT_CACTISPIKE
			and not inflictor.instakill
				return false
			end
		end
	end
end, MT_PLAYER)

addHook("HurtMsg", function(p, inflictor)

	if not inflictor return end			-- got hit by a ghost nice
	if not inflictor.valid return end	-- got hit by a dead thing nice

	if inflictor.type == MT_CACTISPIKE
	and inflictor.source
		if MP_isDead(p)
			print(p.cname.." was killed by "..inflictor.source.player.cname.."'s Cactus Shield spike!")
			return true
		else
			print(p.cname.." was hit by "..inflictor.source.player.cname.."'s Cactus Shield spike!")
			return true
		end
	end

	if not inflictor.player return end	-- not a player rofl

	if inflictor.player.powers[pw_shield] == SH_CACTUSSHIELD
		if MP_isDead(p)
			print(p.cname.." was killed by "..inflictor.player.cname.."'s Cactus Shield!")
			return true
		else
			print(p.cname.." was hit by "..inflictor.player.cname.."'s Cactus Shield!")
			return true
		end
	end
end)

MP_buildMonitor(MT_CACTISHIELDMONITOR, MT_CACTISHIELDICON, S_CCTV, S_CCPOP1, SPR_CCTV, SPR_GRTV, A_CactiShield)

states[S_CACTISHIELD_FRONT1] = {SPR_CSPK, A, 2, nil, 0, 0, S_CACTISHIELD_FRONT2}
states[S_CACTISHIELD_FRONT2] = {SPR_CSPK, B, 2, nil, 0, 0, S_CACTISHIELD_FRONT3}
states[S_CACTISHIELD_FRONT3] = {SPR_CSPK, C, 2, nil, 0, 0, S_CACTISHIELD_FRONT4}
states[S_CACTISHIELD_FRONT4] = {SPR_CSPK, D, 2, nil, 0, 0, S_CACTISHIELD_FRONT1}

states[S_CACTISHIELD_BACK1] = {SPR_CSPK, A, 2, nil, 1, 0, S_CACTISHIELD_BACK2}
states[S_CACTISHIELD_BACK2] = {SPR_CSPK, F, 2, nil, 1, 0, S_CACTISHIELD_BACK3}
states[S_CACTISHIELD_BACK3] = {SPR_CSPK, C, 2, nil, 1, 0, S_CACTISHIELD_BACK4}
states[S_CACTISHIELD_BACK4] = {SPR_CSPK, E, 2, nil, 1, 0, S_CACTISHIELD_BACK1}

states[S_CACTISHIELD_FRONT_B1] = {SPR_CSPK, G, 2, nil, 0, 0, S_CACTISHIELD_FRONT_B2}
states[S_CACTISHIELD_FRONT_B2] = {SPR_CSPK, H, 2, nil, 0, 0, S_CACTISHIELD_FRONT_B3}
states[S_CACTISHIELD_FRONT_B3] = {SPR_CSPK, I, 2, nil, 0, 0, S_CACTISHIELD_FRONT_B4}
states[S_CACTISHIELD_FRONT_B4] = {SPR_CSPK, J, 2, nil, 0, 0, S_CACTISHIELD_FRONT_B1}

states[S_CACTISHIELD_ORB1] = {SPR_PITY, A|FF_TRANS50, 2, nil, 0, 0, S_CACTISHIELD_ORB2}
states[S_CACTISHIELD_ORB2] = {SPR_PITY, B|FF_TRANS50, 2, nil, 0, 0, S_CACTISHIELD_ORB3}
states[S_CACTISHIELD_ORB3] = {SPR_PITY, C|FF_TRANS50, 2, nil, 0, 0, S_CACTISHIELD_ORB4}
states[S_CACTISHIELD_ORB4] = {SPR_PITY, DFF_TRANS50, 2, nil, 0, 0, S_CACTISHIELD_ORB5}
states[S_CACTISHIELD_ORB5] = {SPR_PITY, E|FF_TRANS50, 2, nil, 0, 0, S_CACTISHIELD_ORB6}
states[S_CACTISHIELD_ORB6] = {SPR_PITY, F|FF_TRANS50, 2, nil, 0, 0, S_CACTISHIELD_ORB1}

states[S_CACTISHIELD_BACK_B1] = {SPR_CSPK, G, 2, nil, 1, 0, S_CACTISHIELD_BACK_B2}
states[S_CACTISHIELD_BACK_B2] = {SPR_CSPK, L, 2, nil, 1, 0, S_CACTISHIELD_BACK_B3}
states[S_CACTISHIELD_BACK_B3] = {SPR_CSPK, I, 2, nil, 1, 0, S_CACTISHIELD_BACK_B4}
states[S_CACTISHIELD_BACK_B4] = {SPR_CSPK, K, 2, nil, 1, 0, S_CACTISHIELD_BACK_B1}

states[S_CACTISPIKE] = {SPR_CCPK, A, -1, nil, 0, 0, S_CACTISPIKE}

mobjinfo[MT_CACTISPIKE] = {
	doomednum = -1,
    spawnstate = S_CACTISPIKE,
    spawnhealth = 1000,
    seestate = S_NULL,
    seesound = 0,
    reactiontime = 8,
    attacksound = sfx_None,
    painstate = S_CCTV,
    painchance = 0,
    painsound = sfx_None,
    meleestate = S_NULL,
    missilestate = S_NULL,
    deathstate = S_NULL,
    xdeathstate = S_NULL,
    deathsound = sfx_spkdth,
    speed = 2*FRACUNIT,
    radius = 8*FRACUNIT,
    height = 14*FRACUNIT,
    dispoffset = 0,
    mass = 100,
    damage = 0,
    activesound = sfx_None,
    flags = MF_MISSILE|MF_NOGRAVITY|MF_FLOAT,
    raisestate = 0
}