local dig_dropSpeed = -48*FRACUNIT
local dig_diggingTics = TICRATE*5/2
local dig_minDigTics = dig_diggingTics / 5	-- make sure this is less than dig_diggingTics
local dig_tiredThreshold = 1*TICRATE	-- ^ ditto
local dig_resurfaceHeight = 12*FRACUNIT

function A_DigShield(actor)
	if not actor.target or not actor.target.player
		print("ERROR: A_DigShield: Power up has no target.")
		return
	end

	local p = actor.target.player
	local mo = p.mo

	MP_giveShield(p, SH_DIGSHIELD)
end

addHook("ShouldDamage", function(mo, inflictor, source)
	if (inflictor or source)
	and (mo.dig_diggingtimer)
		return false
	end
end, MT_PLAYER)

local function resetDig(mo)
	local p = mo.player
	
	p.charability2 = skins[mo.skin].ability2
	p.jumpfactor = skins[mo.skin].jumpfactor
	p.pflags = $ & ~(PF_SPINNING|PF_STARTDASH)
	mo.flags = ($ & ~(MF_NOCLIPTHING)) | MF_SHOOTABLE
	mo.flags2 = $ & ~(MF2_DONTDRAW)
	mo.dig_diggingtimer = nil
end

local function digthink(mo)
	local p = mo.player
	local orb = mo.MP_shieldorb
	
	-- dropping state
	if (mo.dig_dropping)
		if (P_IsObjectOnGround(mo))
			-- dig into the ground
			p.jumpfactor = 0
			mo.dig_dropping = nil
			mo.dig_diggingtimer = dig_diggingTics
			orb.flags2 = $|MF2_DONTDRAW
			S_StartSound(mo, sfx_s3k82)
		else
			-- continue dropping
			mo.flags = $|MF_NOCLIPTHING
			P_SetObjectMomZ(mo, dig_dropSpeed)
			P_SpawnGhostMobj(mo)
		end
	end
	
	-- digging state
	if (mo.dig_diggingtimer)
		mo.dig_diggingtimer = $-1
		local timer = mo.dig_diggingtimer
		
		-- conditions to end it
		local timerThreshold = dig_diggingTics - dig_minDigTics
		
		if not (timer)	-- ran out of time
		or not (P_IsObjectOnGround(mo))	-- not grounded
		or (timer < timerThreshold and not (p.cmd.buttons & BT_USE))	-- not holding spin after a certain time
			-- damage stuff around you
			for fuckm in mobjs.iterate()	-- this is really poorly optimized.
				if fuckm == mo continue end
				if R_PointToDist2(mo.x, mo.y, fuckm.x, fuckm.y) > 128*FRACUNIT continue end

				if (fuckm.player or fuckm.flags & MF_SHOOTABLE)
				and fuckm.z - mo.z < 128*FRACUNIT
					P_DamageMobj(fuckm, mo, mo)
				end
			end
			
			-- burst out from the ground
			mo.state = S_PLAY_FALL
			P_SetObjectMomZ(mo, dig_resurfaceHeight)
			S_StartSound(mo, sfx_crumbl)
			S_StopSoundByID(mo, sfx_mindig)
			for i = 1, 32
				local rock = P_SpawnMobj(mo.x + P_RandomRange(-60, 60)*FRACUNIT, mo.y + P_RandomRange(-60, 60)*FRACUNIT, mo.z, MT_THOK)
				rock.momx, rock.momy = mo.momx/2, mo.momy/2
				rock.sprite = SPR_ROIA
				rock.scale = (mo.scale - P_RandomRange(-mo.scale/4+1, mo.scale/4-1))
				rock.fuse = TICRATE
				rock.tics = TICRATE
				rock.sprite = SPR_ROIA
				rock.frame = A				
				rock.flags = MF_SCENERY
				P_SetObjectMomZ(rock, P_RandomRange(4, 15)*FRACUNIT)
			end
			
			resetDig(mo)
			return
		end
		
		-- digging stuff
		p.charability2 = CA2_NONE
		p.weapondelay = 5
		p.pflags = $ & ~(PF_SPINNING|PF_STARTDASH)
		
		mo.state = S_PLAY_WALK
		mo.height = FRACUNIT/10
		mo.flags2 = $|MF2_DONTDRAW
		orb.flags2 = $|MF2_DONTDRAW
		
		-- eyecandies
		if not (leveltime%5) then S_StartSound(mo, sfx_mindig) end
		
		local ground = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
		ground.sprite, ground.frame = SPR_MNUD, F
		ground.tics = 1
		local g = P_SpawnGhostMobj(ground)
		g.scale = mo.scale
		g.destscale = 0
		
		if (timer < dig_tiredThreshold)
			-- close to resurfacing
			local rock = P_SpawnMobj(mo.x + P_RandomRange(-30, 30)*FRACUNIT, mo.y + P_RandomRange(-30, 30)*FRACUNIT, mo.z, MT_THOK)
			rock.flags = MF_SCENERY
			rock.fuse = TICRATE
			rock.tics = TICRATE
			rock.sprite = SPR_ROIA
			rock.frame = A
			rock.scale = mo.scale/3
			rock.momx = FixedMul(mo.scale, P_RandomRange(-7, 7)*FRACUNIT)
			rock.momy = FixedMul(mo.scale, P_RandomRange(-7, 7)*FRACUNIT)
			P_SetObjectMomZ(rock, P_RandomRange(2, 6)*FRACUNIT)
		end
	end
end

local function digSpecial(mo)
	local p = mo.player
	
	p.pflags = $|(PF_THOKKED|PF_SHIELDABILITY)
	mo.flags = $|MF_NOCLIPTHING
	mo.dig_dropping = true
	P_SetObjectMomZ(mo, dig_dropSpeed)
	P_SpawnGhostMobj(mo)
	S_StartSound(mo, sfx_zoom)
	return true
end



local dig_data = {
	orbstate = S_DIG_ORB1,
	hudico = "DIGICO",
	sound = sfx_dig,
	func = digthink,
	special = digSpecial,
	change = resetDig,
}
shieldList[SH_DIGSHIELD] = dig_data

addHook("HurtMsg", function(p, inflictor, source)

	if not inflictor return end
	if not inflictor.valid return end
	if not inflictor.player return end

	if inflictor.player.powers[pw_shield] == SH_DIGSHIELD
	and inflictor.digging
		if MP_isDead(p)
			print(p.cname.." was buried alive by "..inflictor.player.cname.."'s Earth Shield dig attack!")
			return true
		else
			print(p.cname.." was hit by "..inflictor.player.cname.."'s Earth Shield dig attack.")
			return true
		end
	end
end)

MP_buildMonitor(MT_DIGMONITOR, MT_DIGICON, S_DIGTV, S_DIGPOP1, SPR_EART, SPR_EART, A_DigShield)

states[S_DIG_ORB1] = {SPR_EARS, A|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB2}
states[S_DIG_ORB2] = {SPR_EARS, B|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB3}
states[S_DIG_ORB3] = {SPR_EARS, C|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB4}
states[S_DIG_ORB4] = {SPR_EARS, D|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB5}
states[S_DIG_ORB5] = {SPR_EARS, E|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB6}
states[S_DIG_ORB6] = {SPR_EARS, F|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB7}
states[S_DIG_ORB7] = {SPR_EARS, G|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB8}
states[S_DIG_ORB8] = {SPR_EARS, H|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB9}
states[S_DIG_ORB9] = {SPR_EARS, I|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB10}
states[S_DIG_ORB10] = {SPR_EARS, J|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB11}
states[S_DIG_ORB11] = {SPR_EARS, K|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB12}
states[S_DIG_ORB12] = {SPR_EARS, L|TR_TRANS30, 2, nil, 0, 0, S_DIG_ORB1}