-- TODO: Sam: spin ability for psn shield?

function A_PoisonShield(actor)
	if not actor.target or not actor.target.player
		print("ERROR: A_PoisonShield: Power up has no target.")
		return
	end

	local p = actor.target.player
	MP_giveShield(p, SH_POISONSHIELD)
end

-- main thinker

addHook("MobjThinker", function(mo)
	if mo and mo.valid
		mo.scale = $ - FRACUNIT/90
	end
end, MT_POISONTRAIL)

local function efct_poison(mo, inflictor, time)
	local p = mo.player
	if not (time%4)
		for i = 1,2
			local psn = P_SpawnMobj(mo.x+P_RandomRange(-15,15)*FRACUNIT, mo.y+P_RandomRange(-15,15)*FRACUNIT, mo.z+P_RandomRange(0,15)*FRACUNIT, MT_THOK)
			psn.frame = A
			psn.color = SKINCOLOR_MAGENTA
			psn.scale = mo.scale/2
			psn.destscale = mo.scale/99
			psn.momz = P_RandomRange(5, 12)*FRACUNIT
			psn.tics = TICRATE
			S_StartSound(mo, sfx_psned, p)
		end
	end
	if not (time%TICRATE)
		if mo.health
			p.rings = max(0, $-2)
			S_StartSound(mo, sfx_psnr, p)
		end
		if inflictor and inflictor.valid
			local mult = p.rings == 0 and 2 or 1
			P_AddPlayerScore(inflictor.player, 5*mult)
		end
	end
	if mo.eflags & MFE_UNDERWATER
		MP_loseEffect(p, ef_poison)
	end
end

-- hud

local function psn_hud(v, p, c, t)
	if not p.mo return end
	local mo = p.mo
	if leveltime % 10 < 5
		v.draw(hudinfo[HUD_RINGS].x, hudinfo[HUD_RINGS].y, v.cachePatch("PSN_RING"), V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER)
	end
end

local psnefct_data = {
	time = TICRATE*10,		-- this effects last for...?
	func = efct_poison,		-- func that runs on loop while the effect is active
	get = nil,				-- func that runs when you GET the effect
	lose = nil,				-- func that runs when you LOSE the effect (by effect remove or timer running out)
	hit = nil,				-- func that runs when you get hit with the effect active
	hudico = nil,			-- display a hud icon along with the effect (useful especially if it's meant to be timed.)
	hudfunc = psn_hud,		-- runs a hud function for the effect
}
effectList[ef_poison] = psnefct_data

local function poisonShield_main(mo)
	if P_IsObjectOnGround(mo)
	and leveltime%2 == 0
	and mo.health
	and FixedMul(mo.scale, mo.player.speed) > FixedMul(mo.player.normalspeed/2, mo.scale)
		local trail = P_SpawnMobj(mo.x, mo.y, mo.z, MT_POISONTRAIL)
		trail.target = mo	-- this will avoid self hurting
		trail.scale = mo.scale*3/2
		trail.radius = mo.radius*3/2 - mo.radius
		trail.spawnby = mo.player
		trail.fuse = 90
	elseif mo.eflags & MFE_UNDERWATER
		MP_loseShield(mo.player, true)
	end
end

local function poisonShield_special(mo)
	
	for i = 1, 16 do
		local s = P_SpawnMobj(mo.x, mo.y, mo.z, MT_POISONPROJ)
		s.momx = P_RandomRange(-20, 20)*mo.scale
		s.momy = P_RandomRange(-20, 20)*mo.scale
		s.momz = P_RandomRange(-10, 20)*mo.scale
		
		s.scale = mo.scale
		s.target = mo
	end
	
	mo.player.pflags = $|PF_SHIELDABILITY|PF_THOKKED
	S_StartSound(mo, sfx_ghit)
end

local poisonShield_data = {
	orbstate = S_PSN_ORB1,
	hudico = "PSN_ICO",
	sound = sfx_s3kbfs,
	func = poisonShield_main,
	special = poisonShield_special,
}
shieldList[SH_POISONSHIELD] = poisonShield_data

-- colliding with shield player

addHook("MobjCollide", function(mo, touch)

    if not touch return end                        -- no item wtf
    if not touch.player return end                -- not even a fucking player
    if not (touch.player.powers[pw_shield] == SH_POISONSHIELD) return end        -- no shield fuck you: EDIT 4/9 FORGOT BRACKETS KILL ME
	if touch.player and touch.player.powers[pw_shield] == SH_TITANIUMSHIELD return end	
    if touch.z > mo.z+mo.height return end        -- too high
    if touch.z+touch.height < mo.z return end    -- too low
    if mo.player and (mo.player.powers[pw_super] or mo.player.powers[pw_invulnerability]) return end
	if friendPlayer(mo.player, touch.player) return end

	if not (mo.player.powers[pw_shield] == SH_POISONSHIELD or mo.player.powers[pw_shield] == SH_TITANIUMSHIELD)
    and not mo.MP_effects[ef_poison]
		for i = ef_CUTOFF, #effectList
			if mo.MP_effects[i]
				return
			end	
		end
        print(mo.player.cname.." was poisoned by "..touch.player.cname.."!")
        MP_giveEffect(mo.player, ef_poison, touch)
    end
end, MT_PLAYER)

-- colliding with pudge....

addHook("MobjMoveCollide", function(mo, touch)

	if not touch return end
	if touch.player and touch.player.powers[pw_shield] == SH_POISONSHIELD return end
	if touch.player and touch.player.powers[pw_shield] == SH_TITANIUMSHIELD return end
	if touch == mo.target return end
	if touch.z > mo.z+mo.height return end		-- too high
	if touch.z+touch.height < mo.z return end	-- too low
    if mo.player and (mo.player.powers[pw_super] or mo.player.powers[pw_invulnerability]) return end
	if not mo.target or not mo.target.valid return end

	if touch.flags & MF_ENEMY
		P_DamageMobj(touch, mo, mo.target)	-- doit
		return
	end

	if touch.player
	and not touch.MP_effects[ef_poison]
	and not friendPlayer(touch.player, mo.target.player)

		for i = ef_CUTOFF, #effectList
			if touch.MP_effects[i]
				return
			end	
		end
		
		if mo.target.player
			print(touch.player.cname.." was poisoned by "..mo.target.player.cname.."'s poison trail!")
		end
		MP_giveEffect(touch.player, ef_poison, mo.target)
	end

end, MT_POISONTRAIL)

addHook("MobjMoveCollide", function(mo, touch)

	if not touch return end
	if touch.player and touch.player.powers[pw_shield] == SH_POISONSHIELD return end
	if touch.player and touch.player.powers[pw_shield] == SH_TITANIUMSHIELD return end
	if touch == mo.target return end
	if touch.z > mo.z+mo.height return end		-- too high
	if touch.z+touch.height < mo.z return end	-- too low
	if touch.player and touch.player.powers[pw_super] return end
	if not mo.target or not mo.target.valid return end

	if touch.flags & MF_ENEMY
		P_DamageMobj(touch, mo, mo.target)	-- doit
		return
	end

	if touch.player
	and not touch.MP_effects[ef_poison]
	and not friendPlayer(touch.player, mo.target.player)
		if not G_RingSlingerGametype() return end
		if mo.target.player
			print(touch.player.cname.." was poisoned by "..mo.target.player.cname.."'s poison trail!")
		end
		MP_giveEffect(touch.player, ef_poison, mo.target)
	end

end, MT_POISONPROJ)

addHook("MobjThinker", function(mo)
	
	local visual = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
	visual.tics = 2
	visual.scale = FRACUNIT + FixedMul(FRACUNIT/4, cos(leveltime*35*ANG1))
	visual.color = SKINCOLOR_MAGENTA
	
	mo.momz = $ + P_GetMobjGravity(mo)*2
	
	local g = P_SpawnGhostMobj(visual)
	g.tics = 3
	
	if P_IsObjectOnGround(mo)
		local t = P_SpawnMobj(mo.x, mo.y, mo.z, MT_POISONTRAIL)
		t.scale = mo.scale*3/2
		t.target = mo.target
		t.fuse = 90
		P_RemoveMobj(mo)
		return
	end
end, MT_POISONPROJ)

MP_buildMonitor(MT_POISONMONITOR, MT_POISONICON, S_PSNTV, S_PSNPOP1, SPR_PSTV, SPR_PSTV, A_PoisonShield)

states[S_PSN_ORB1] = {SPR_PSNS, A|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB2}
states[S_PSN_ORB2] = {SPR_PSNS, B|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB3}
states[S_PSN_ORB3] = {SPR_PSNS, C|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB4}
states[S_PSN_ORB4] = {SPR_PSNS, D|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB5}
states[S_PSN_ORB5] = {SPR_PSNS, E|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB6}
states[S_PSN_ORB6] = {SPR_PSNS, F|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB7}
states[S_PSN_ORB7] = {SPR_PSNS, G|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB8}
states[S_PSN_ORB8] = {SPR_PSNS, H|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB9}
states[S_PSN_ORB9] = {SPR_PSNS, I|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB10}
states[S_PSN_ORB10] = {SPR_PSNS, J|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB11}
states[S_PSN_ORB11] = {SPR_PSNS, K|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB12}
states[S_PSN_ORB12] = {SPR_PSNS, L|TR_TRANS50, 2, nil, 0, 0, S_PSN_ORB1}

states[S_POISONTRAIL1] = {SPR_PSNP, A, 4, nil, 0, 0, S_POISONTRAIL2}
states[S_POISONTRAIL2] = {SPR_PSNP, B, 4, nil, 0, 0, S_POISONTRAIL3}
states[S_POISONTRAIL3] = {SPR_PSNP, C, 4, nil, 0, 0, S_POISONTRAIL4}
states[S_POISONTRAIL4] = {SPR_PSNP, D, 4, nil, 0, 0, S_POISONTRAIL1}

mobjinfo[MT_POISONTRAIL] = {
	doomednum = -1,
    spawnstate = S_POISONTRAIL1,
    spawnhealth = 1000,
    seestate = S_NULL,
    seesound = sfx_None,
    reactiontime = 8,
    attacksound = sfx_None,
    painstate = S_NULL,
    painchance = 0,
    painsound = sfx_None,
    meleestate = S_NULL,
    missilestate = S_NULL,
    deathstate = S_NULL,
    xdeathstate = S_NULL,
    deathsound = sfx_pop,
    speed = 0,
    radius = 16*FRACUNIT,
    height = 5*FRACUNIT,
    dispoffset = 0,
    mass = 100,
    damage = 0,
    activesound = sfx_None,
    flags = MF_NOCLIPHEIGHT|MF_NOGRAVITY,
    raisestate = 0
}

mobjinfo[MT_POISONPROJ] = {
	doomednum = -1,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1000,
	radius = 16 << FRACBITS,
	height = 32 << FRACBITS,
	flags = MF_RUNSPAWNFUNC,
}