function A_ThunderShield(actor)
	if not actor.target or not actor.target.player
		print("ERROR: A_ThunderShield: Power up has no target.")
		return
	end
	local p = actor.target.player
	MP_giveShield(p, SH_THUNDERSHIELD)
end

local function ThunderBlast(mo)
	-- ZIODYNE!
	local strength = max(256*FRACUNIT, mo.z - mo.floorz)
	strength = min(1366*FRACUNIT, $)
	local thok = P_SpawnMobj(mo.x, mo.y, mo.floorz, MT_THOK)
	thok.fuse = 35
	S_StartSound(thok, sfx_zio3)
	thok.state = S_INVISIBLE
	S_StartSound(mo, sfx_zio3)
	for i = 1, 16
		local tt = P_SpawnMobj(mo.x, mo.y, mo.floorz, MT_THOK)
		tt.state = S_SSPK1
		local an = ANGLE_22h*(i-1)
		P_InstaThrust(tt, an, strength/16)
	end
	for mobj in mobjs.iterate()
		if mobj and mobj.valid and (mobj.flags & (MF_ENEMY|MF_BOSS) or mobj.type == MT_PLAYER) and R_PointToDist2(mo.x, mo.y, mobj.x, mobj.y) < strength and abs((mo.floorz - mobj.z)) < strength/2
			if mobj.player and (friendPlayer(mobj.player, mo.player) or mobj.player.powers[pw_shield] == SH_THUNDERSHIELD) continue end
			P_DamageMobj(mobj, mo, mo)
		end
	end
end

local function thunderThink(mo)
	local p = mo.player
	
	-- lose it underwater
	if (mo.eflags & MFE_UNDERWATER)
		S_StartSound(mo, sfx_becrsh)
		MP_loseShield(p, true)
		P_FlashPal(p, 1, 2)
		return
	end
	
	-- particles
	if not (leveltime%3)
		local spark = MP_spawnParticles(mo, MT_THUNDERSPARK, mo.x+P_RandomRange(30, -30)*mo.scale, mo.y+P_RandomRange(-30, 30)*mo.scale)
		if (spark and spark.valid)
			spark.destscale = 1
			spark.fuse = 20
			P_SetObjectMomZ(spark, -3*spark.scale)
		end
	end
end

local function thunderSpecial(mo)
	local p = mo.player
	
	p.pflags = ($ & ~(PF_JUMPED)) | (PF_THOKKED|PF_SHIELDABILITY)
	mo.state = S_PLAY_FALL
	mo.momx, mo.momy = 0, 0
	P_SetObjectMomZ(mo, 5*FRACUNIT)
	S_StartSound(mo, sfx_thndr)
	ThunderBlast(mo)
	MP_spawnLightning(mo, SKINCOLOR_COBALT, SKINCOLOR_SAPPHIRE)
	return true
end

local tshield_data = {
	orbstate = S_THUND_ORB1,
	hudico = "THUNDICO",
	sound = sfx_beelec,
	func = thunderThink,
	special = thunderSpecial,
}
shieldList[SH_THUNDERSHIELD] = tshield_data

local function inSparkSec(mo)
	local sec = mo.subsector.sector
	local numlines = #sec.lines

	if sec.special == 4
		return true
	end

	for i=1,numlines

		local v = sec.lines[i]
		if v

			if v.frontsector and v.frontsector.special == 4
			or v.backsector and v.backsector.special == 4
				return true
			end
		end
	end
end


addHook("ShouldDamage", function(mo, inflictor, source)

	if not inflictor return end

	if mo and mo.valid
	local p = mo.player

		if not p return end

		if mo.thundershield and not inflictor
			if P_PlayerTouchingSectorSpecial(p, 1, 4)
			or inSparkSec(mo)
				return false
			end
		end
	end
end, MT_PLAYER)

addHook("HurtMsg", function(p, inflictor, source)

	if not inflictor return end
	if not inflictor.valid return end
	if not inflictor.player return end

	if inflictor.player.powers[pw_shield] == SH_THUNDERSHIELD
	and inflictor.used_thund
		if MP_isDead(p)
			print(p.cname.." was fried by "..inflictor.player.cname.."'s Thunder Shield blast!")
			return true
		else
			print(p.cname.." was shocked by "..inflictor.player.cname.."'s Thunder Shield blast.")
			return true
		end
	end
end)

MP_buildMonitor(MT_THUNDERMONITOR, MT_THUNDERICON, S_THDRTV, S_THDRPOP1, SPR_THDR, SPR_THDR, A_ThunderShield)

states[S_THUND_ORB1] = {SPR_TDSH, A|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB2}
states[S_THUND_ORB2] = {SPR_TDSH, B|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB3}
states[S_THUND_ORB3] = {SPR_TDSH, C|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB4}
states[S_THUND_ORB4] = {SPR_TDSH, D|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB5}
states[S_THUND_ORB5] = {SPR_TDSH, E|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB6}
states[S_THUND_ORB6] = {SPR_TDSH, F|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB7}
states[S_THUND_ORB7] = {SPR_TDSH, G|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB8}
states[S_THUND_ORB8] = {SPR_TDSH, H|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB9}
states[S_THUND_ORB9] = {SPR_TDSH, I|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB10}
states[S_THUND_ORB10] = {SPR_TDSH, J|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB11}
states[S_THUND_ORB11] = {SPR_TDSH, K|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB12}
states[S_THUND_ORB12] = {SPR_TDSH, L|TR_TRANS50, 2, nil, 0, 0, S_THUND_ORB1}

states[S_ZIO1] = {SPR_EZIO, A, 2, nil, 0, 0, S_ZIO2}
states[S_ZIO2] = {SPR_EZIO, A|TR_TRANS30, 2, nil, 0, 0, S_ZIO3}
states[S_ZIO3] = {SPR_EZIO, A|TR_TRANS50, 2, nil, 0, 0, S_ZIO4}
states[S_ZIO4] = {SPR_EZIO, A|TR_TRANS70, 2, nil, 0, 0, S_NULL}

states[S_ZIO21] = {SPR_EZIO, B, 2, nil, 0, 0, S_ZIO22}
states[S_ZIO22] = {SPR_EZIO, B|TR_TRANS30, 2, nil, 0, 0, S_ZIO23}
states[S_ZIO23] = {SPR_EZIO, B|TR_TRANS50, 2, nil, 0, 0, S_ZIO24}
states[S_ZIO24] = {SPR_EZIO, B|TR_TRANS70, 2, nil, 0, 0, S_NULL}

states[S_THUNDERSPARK] = {SPR_TUPA, A, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK1}
states[S_THUNDERSPARK1] = {SPR_TUPA, B, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK2}
states[S_THUNDERSPARK2] = {SPR_TUPA, C, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK3}
states[S_THUNDERSPARK3] = {SPR_TUPA, D, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK4}
states[S_THUNDERSPARK4] = {SPR_TUPA, E, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK5}
states[S_THUNDERSPARK5] = {SPR_TUPA, F, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK6}
states[S_THUNDERSPARK6] = {SPR_TUPA, G, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK7}
states[S_THUNDERSPARK7] = {SPR_TUPA, H, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK8}
states[S_THUNDERSPARK8] = {SPR_TUPA, G, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK9}
states[S_THUNDERSPARK9] = {SPR_TUPA, F, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK10}
states[S_THUNDERSPARK10] = {SPR_TUPA, E, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK11}
states[S_THUNDERSPARK11] = {SPR_TUPA, D, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK12}
states[S_THUNDERSPARK12] = {SPR_TUPA, C, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK13}
states[S_THUNDERSPARK13] = {SPR_TUPA, B, 2, A_SparkleThink, 0, 0, S_THUNDERSPARK14}
states[S_THUNDERSPARK14] = {SPR_TUPA, A, 2, A_SparkleThink, 0, 0, S_NULL}

mobjinfo[MT_THUNDERSPARK] = {
	doomednum = -1,
    spawnstate = S_THUNDERSPARK,
    spawnhealth = 1000,
    seestate = S_NULL,
    seesound = 0,
    reactiontime = 8,
    attacksound = sfx_None,
    painchance = 0,
    painsound = sfx_None,
    meleestate = S_NULL,
    missilestate = S_NULL,
    deathstate = S_NULL,
    xdeathstate = S_NULL,
    deathsound = sfx_becrsh,
    speed = 2*FRACUNIT,
    radius = 8*FRACUNIT,
    height = 14*FRACUNIT,
    dispoffset = 0,
    mass = 100,
    damage = 0,
    activesound = sfx_None,
    flags = MF_NOBLOCKMAP|MF_NOGRAVITY|MF_NOCLIP|MF_SCENERY|MF_NOCLIPHEIGHT,
    raisestate = 0
}