-- honestly no need to make it an effect given that 90% of the thinker is inside the shell itself.

local function highestScore(pl)
	local hiscore = -1
	local hiscoreplayer

	for p in players.iterate do
		if not p.mo continue end
		if p ~= pl and friendPlayer(p, pl) continue end	-- don't target out friends, but we may still target ourselves.
		if not p.mo.valid continue end

		if p.score > hiscore
			hiscore = p.score
			hiscoreplayer = p
		end
	end
	return hiscoreplayer
end

function A_BlueShellTV(actor)
	if not actor.target or not actor.target.player
		print("ERROR: A_BlueShellTV: Power up has no target.")
		return
	end

	local p = actor.target.player
	local mo = p.mo

	-- get player

	local targetplayer = highestScore(p)

	if targetplayer and targetplayer.mo and targetplayer.mo.valid then targetplayer = targetplayer.mo end

	-- spawn blue shell

	local shell = P_SpawnMobj(actor.x, actor.y, actor.z, MT_BLUESHELL)
	shell.tp = targetplayer		-- chase hi score player
	shell.target = mo				-- refer to the player that got the monitor to give points
	targetplayer.shellwarning = TICRATE*2
	S_StartSound(nil, sfx_shell, targetplayer.player)
end

addHook("ThinkFrame", do
	for p in players.iterate
		if not p.mo continue end
		if not p.mo.valid continue end
		if not p.mo.shellwarning continue end

		if p.mo.shellwarning then p.mo.shellwarning = $-1 end
	end
end)

-- shell thinker

addHook("MobjThinker", function(mo)
	if not mo or not mo.valid return end
	mo.state = S_THOK
	mo.tics = -1
	mo.sprite = SPR_SHEL
	mo.frame = F +leveltime%3
	if not mo.kaboom then mo.kaboom = 0 end

	if not mo.tp or not mo.tp.valid
		P_RemoveMobj(mo)
		return
	end

	if mo.tp and mo.tp.valid

		if (R_PointToDist2(mo.x, mo.y, mo.tp.x, mo.tp.y) > FRACUNIT*80 or mo.z < mo.tp.z)
		and not mo.kaboom

			local thing = P_SpawnMobj(mo.tp.x, mo.tp.y, mo.tp.z+mo.tp.height+20*FRACUNIT, MT_THOK)
			thing.state = S_INVISIBLE
			thing.tics = 2
			mo.tracer = thing
			A_HomingChase(mo, 61*FRACUNIT, 1)
		else
			if not mo.kaboom then mo.kaboom = 1 end
			mo.flags = $ & ~MF_NOGRAVITY
			if mo.kaboom == 1 then
				P_SetObjectMomZ(mo, FRACUNIT*10)
				mo.momx = mo.tp.momx
				mo.momy = mo.tp.momy
			end
			P_SetObjectMomZ(mo, -gravity*5, true)
			mo.kaboom = $+1

			if mo.z < mo.floorz
				-- explosion

				local truc = P_SpawnMobj(mo.x, mo.y, mo.z, MT_BLUXPL)
				truc.color = SKINCOLOR_BLUE
				truc.scale = FRACUNIT*3
				S_StartSound(truc, sfx_megid3)

				mo.savetarget = mo.target or $
				mo.target = nil

				for p in players.iterate
					if not p.mo continue end
					local pmo = p.mo

					if R_PointToDist2(mo.x, mo.y, pmo.x, pmo.y) < FRACUNIT*400
						mo.flags = $|MF_MISSILE
						if savetarget == pmo
							pmo.selfharm = true
							mo.target = nil
							P_DamageMobj(pmo, mo)
						else
							mo.target = savetarget
							P_DamageMobj(pmo, mo, savetarget)
						end
						P_SetObjectMomZ(pmo, 19*FRACUNIT)
					end
				end

				P_RemoveMobj(mo)
				return

			end
		end
	end
end, MT_BLUESHELL)


local function shellwarning(v, p)
	if p and p.valid and p.mo and p.mo.valid and p.mo.shellwarning

		if leveltime%10 < 5
			v.draw(160, 70, v.cachePatch("SHELLICO"))
		end
	end
end

hud.add(shellwarning)

addHook("HurtMsg", function(p, inflictor, source)

    if not inflictor return end
	if inflictor.type ~= MT_BLUESHELL return end
    if not inflictor.valid return end
    if not inflictor.savetarget or not inflictor.savetarget.player return end

	inflictor.target = inflictor.savetarget

	if p.mo == inflictor.savetarget
		p.mo.selfharm = nil
		if MP_isDead(p)
			print(p.cname.." was killed by their own Blue Shell. Baka.")
			return true
		else
			print(p.cname.." was hit by their own Blue Shell.")
			return true
		end
	else
		if MP_isDead(p)
			print(inflictor.savetarget.player.cname.." killed "..p.cname.." with their Blue Shell!")
			return true
		else
			print(inflictor.savetarget.player.cname.." hit "..p.cname.." with their Blue Shell.")
			return true
		end
	end
end)

mobjinfo[MT_BLUXPL] = {
	doomednum = -1,
	spawnstate = S_BLUXPL1,
	spawnhealth = 1,
	seestate = S_NULL,
	seesound = sfx_None,
	reactiontime = 0,
	attacksound = SFX_None,
	painstate = S_NULL,
	painsound = sfx_None,
	meleestate = S_NULL,
	missilestate = S_NULL,
	deathstate = S_NULL,
	deathsound = sfx_None,
	speed = 0,
	radius = 32*FRACUNIT,
	height = 64*FRACUNIT,
	dispoffset = 2,
	mass = 200,
	damage = MT_NULL,
	activesound = sfx_None,
	flags = MF_NOGRAVITY|MF_NOCLIPTHING,
	raisestate = MT_NULL
}

states[S_BLUXPL1] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|A,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL2
}
states[S_BLUXPL2] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|B,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL3
}
states[S_BLUXPL3] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|C,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL4
}
states[S_BLUXPL4] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|D,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL5
}
states[S_BLUXPL5] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|E,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL6
}
states[S_BLUXPL6] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|F,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL7
}
states[S_BLUXPL7] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|G,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL8
}
states[S_BLUXPL8] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|H,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL9
}
states[S_BLUXPL9] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|I,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL10
}
states[S_BLUXPL10] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|J,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL11
}
states[S_BLUXPL11] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|K,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL12
}
states[S_BLUXPL12] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|L,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL13
}
states[S_BLUXPL13] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|M,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL14
}
states[S_BLUXPL14] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|N,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL15
}
states[S_BLUXPL15] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|O,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL16
}
states[S_BLUXPL16] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|P,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL17
}
states[S_BLUXPL17] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|Q,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL18
}
states[S_BLUXPL18] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|R,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL19
}
states[S_BLUXPL19] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|S,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL20
}
states[S_BLUXPL20] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|T,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_BLUXPL21
}
states[S_BLUXPL21] = {
	sprite = SPR_OXPL,
	frame = FF_FULLBRIGHT|U,
	tics =   1,
	action = None,
	var1 = 0,
	var2 = 0,
	nextstate = S_NULL
}

MP_buildMonitor(MT_BSHELLMONITOR, MT_BSHELLICON, S_BSHELLTV, S_BSHELLPOP1, SPR_SHEL, SPR_SHEL, A_BlueShellTV)

mobjinfo[MT_BLUESHELL] = {
	doomednum = -1,
    spawnstate = S_SHELL,
    spawnhealth = 1,
    seestate = S_NULL,
    seesound = sfx_None,
    reactiontime = 8,
    attacksound = sfx_None,
    painstate = 0,
    painchance = 0,
    painsound = sfx_None,
    meleestate = S_NULL,
    missilestate = S_NULL,
    deathstate = 0,
    xdeathstate = S_NULL,
    deathsound = sfx_pop,
    speed = 0,
    radius = 16*FRACUNIT,
    height = 48*FRACUNIT,
    dispoffset = 0,
    mass = 100,
    damage = 1,
    activesound = sfx_None,
    flags = MF_NOGRAVITY|MF_NOCLIPHEIGHT|MF_NOCLIP,
    raisestate = MT_THOK
}